/* voltmeter_display
 *
 * Az A4 analóg bemenetre kötött feszültséget
 * az 5 V-os tápfeszültséghez képest mérjük
 * (itt most egy fényérzékeny osztó kimenetén).
 * A mérést 50-szer ismételjük, s az eredményt átlagoljuk.
 * Az átlagolt eredményt a Multifunkciós kártya 
 * hétszegmenses kijelzőére kiíratjuk.
 * A multiplex kijelzés frissítésének ütemezéséhez 
 * az MsTimer2 programkönyvtárat használjuk.
 * 
 * Hardver követelmények:
 * - Arduino Uno kártya
 * - Multifunkciós kártya
 * - Fényérzékeny ellenállás (LDR) A4 és GND közé kötve
 * - A J1 átkötés legyen zárva (10k felhúzés az A4 pontra)
 * 
 * GND --/\/\/\----+--/\/\/\---- 5V
 *        L D R   A4   10k
 */
#include <MsTimer2.h> 

/* A kijelzőt vezérlő lábak definiálása */
#define LATCH_DIO 4
#define CLK_DIO 7
#define DATA_DIO 8

/* Szegmensrajzolat a 0-9 számjegyekhez */
const byte SEGMENT_MAP[] = {0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, 0X80, 0X90};
/* Az 1-4 számjegyek kiválasztása */
const byte SEGMENT_SELECT[] = {0xF1, 0xF2, 0xF4, 0xF8};
byte segment_data[] = {0xFF, 0xFF, 0xFF, 0xFF}; // adattároló
byte counter = 50;                              // Hány mérést végezzünk
byte idx = 0;                                   // Kijelzés számlálója
long nexttime;                                  // Időbélyeg mérés indításhoz
long sum = 0;                                   // Összegző változó átlagoláshoz

void setup () {
  analogReference(DEFAULT);
  /* Vezérlő lábak konfigurálása */
  pinMode(LATCH_DIO, OUTPUT);
  pinMode(CLK_DIO, OUTPUT);
  pinMode(DATA_DIO, OUTPUT);
  nexttime = millis();
  MsTimer2::set(5, refresh_display);            // 5 ms period
  MsTimer2::start();
}

/* Main program */
void loop() {
  long voltage;
  if (millis() > nexttime) {          // TASK1 100 ms-onként fut le
    sum += analogRead(A4);
    if (--counter == 0) {
      voltage = sum * 100 / 1024;    // Feszültség mV-ban (sum: 50 mérés összege)
      sum = 0;                       // Töröljük az összegző változót
      counter = 50;                  // 50 konverzió eredményét átlagoljuk
      segment_data[0] = SEGMENT_MAP[voltage / 1000];
      segment_data[1] = SEGMENT_MAP[(voltage / 100) % 10];
      segment_data[2] = SEGMENT_MAP[(voltage / 10) % 10];
      segment_data[3] = SEGMENT_MAP[voltage % 10];    
    }
    nexttime += 10;                  // 10 ms-onként végzünk konverziót
  }
}

/* A soron következő számjegy megjelenítése */
void refresh_display() {
  byte i = idx++ & 0x3;
  digitalWrite(LATCH_DIO, LOW);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, segment_data[i]);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, SEGMENT_SELECT[i] );
  digitalWrite(LATCH_DIO, HIGH);
}
