/* voltmeter_display2
 *
 * Az A4 analóg bemenetre kötött feszültséget
 * az 5 V-os tápfeszültséghez képest mérjük
 * (itt most egy fényérzékeny osztó kimenetén).
 * A mérést 50-szer ismételjük, s az eredményt átlagoljuk.
 * Az átlagolt eredményt a Multifunkciós kártya 
 * hétszegmenses kijelzőére kiíratjuk.
 * 
 * A multiplex kijelzés frissítésének ütemezéséhez 
 * Timer2-t regiszter szinten programozzuk.
 * 
 * A főprogramban ráadásul "életjelző" (heartbeat)
 * a beépített LED-et villogtatjuk, így a LED jelzi,
 * hogy fut a program.
 * 
 * Hardver követelmények:
 * - Arduino Uno kártya
 * - Multifunkciós kártya
 * - Fényérzékeny ellenállás (LDR) A4 és GND közé kötve
 * - A J1 átkötés legyen zárva (10k felhúzés az A4 pontra)
 * 
 * GND --/\/\/\----+--/\/\/\---- 5V
 *        L D R   A4   10k
 */

/* A kijelzőt vezérlő lábak definiálása */
#define LATCH_DIO 4
#define CLK_DIO   7
#define DATA_DIO  8
/* Heartbeat LED */
#define ledPin   LED_BUILTIN

/* Szegmensrajzolat a 0-9 számjegyekhez */
const byte SEGMENT_MAP[] = {0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, 0X80, 0X90};

/* Az 1-4 számjegyek kiválasztása */
const byte SEGMENT_SELECT[] = {0xF1, 0xF2, 0xF4, 0xF8};

volatile byte segment_data[] = {0,0,0,0}; // adattároló buffer
byte counter = 50;                        // Ennyi mérést átlagolunk
byte idx = 0;                             // Kijelzés számlálója
byte ledstate = 0;                        // Heartbeat LED állapota
long nexttime;                            // Időbélyeg mérés indításhoz
long blinktime;                           // Időbélyeg hearbeat LED átkapcsoláshoz
long sum = 0;                             // Összegző változó átlagoláshoz


void setup() {
  pinMode(ledPin, OUTPUT);
  analogReference(DEFAULT);
  /* Vezérlő lábak konfigurálása */
  pinMode(LATCH_DIO, OUTPUT);
  pinMode(CLK_DIO, OUTPUT);
  pinMode(DATA_DIO, OUTPUT);
  nexttime = millis();
  blinktime = millis();
  /* Timer2 konfigurálás */
  TIMSK2 = (TIMSK2 & B11111110) | 0x01;   // TIM2_OVF meszakítás engedélyezés
  TCCR2B = (TCCR2B & B11111000) | 0x05;   // előosztó = 1 : 128
}


void loop() {
  long thistime = millis();
  if (thistime > nexttime) {              // TASK1 100 ms-onként fut le
    sum += analogRead(A4);                // Újabb mérés indítása
    if (--counter == 0) {                 // Ha összegyűlt 50 adat...
      long voltage = sum * 100 / 1024;    // Feszültség mV-ban 
      sum = 0;                            // Töröljük az összegző változót
      counter = 50;                       // újabb 50 márást jegyzünk elő
      /* Eredmény kiírása a bufferbe */
      segment_data[0] = SEGMENT_MAP[voltage / 1000] & 0x7F; // Tizedesponttal...
      segment_data[1] = SEGMENT_MAP[(voltage / 100) % 10];
      segment_data[2] = SEGMENT_MAP[(voltage / 10) % 10];
      segment_data[3] = SEGMENT_MAP[voltage % 10];
    }
    nexttime += 10;                       // 10 ms-onként végzünk konverziót
  }

  if (thistime > blinktime ) {            // TASK2: heartbeat LED villogtatása
    blinktime += 1000;                    // következő állapotváltás 1 s múlva
    ledstate = !ledstate;                 // állapotváltás
    digitalWrite(ledPin, ledstate);       // állapotváltás érvényesítése
  }
}

/* Timer2 túlcsordulás megszakításkiszolgáló eljárás 
 * amelyben felvillantjuk a számkijelző egyik számjegyét
 */
ISR(TIMER2_OVF_vect) {
  byte i = idx++ & 0x3;
  digitalWrite(LATCH_DIO, LOW);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, segment_data[i]);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, SEGMENT_SELECT[i] );
  digitalWrite(LATCH_DIO, HIGH);
}
